import pyvisa
import numpy as np
# import pkg_resources


class Keithley2007:
        
    """
    Utility
    """
    scripts = {}
    
    def __init__(self, beep_success=True, visa_backend="", check_front_switch=True):
        rm = pyvisa.ResourceManager(visa_backend)
        resources = rm.list_resources()
        if len(resources) < 1:
            raise Exception("No resources found.")
        elif len(resources) == 1:
            print(f"Opening the only resource found: {resources[0]}")
            self. = rm.open_resource(resources[0])
            if beep_success: keithley.write("beeper.beep(0.5, 1000)")
            return keithley
        elif len(resources) > 1:
            print(f"Resources:")
            for i, r in enumerate(resources):
                print(f"{i:02}: {r}")
            instr = int(input("Select an instrument: "))
            self.instr = rm.open_resource(resources[instr])
            if beep_success: self.beep()
        if check_front_switch:
            self._check_front_input_selected()
        
    def _check_front_input_selected(self):
        switch = self.instr.query("SYSTem:FRSwitch?").strip("\n")
        if switch != "1":
            raise Exception("The Keithley's INPUT switch must select the [F]ront inputs")
        return switch
        

    def run_script(self, script_path, verbose=False):
        """
        Run a script from the host on the instrument
        @param script_path : full path to the script
        """
        with open(script_path, "r") as file:
            script = file.read()
        if verbose: print(f"Running script: {script_path}")
        self.instr.write(script)
    
    def beep(self, length=0.5, pitch=1000):
        """The Keithley2700 cant beep on command :("""
        pass
    
    def __del__(self):
        """Properly close the instrument connection"""
        self.instr.close()

    
    
    def reset(self, verbose=False):
        """
        Reset smua and its buffers
        @param instr : pyvisa instrument
        """
        self.run_script(self.instr, scripts["smua_reset"], verbose=verbose)
        run_lua(self.instr, scripts["buffer_reset"], verbose=verbose)
    

    
    def get_buffer_size(instr, buffer_nr=1):
        n = instr.query(f"print({get_buffer_name(buffer_nr)}.n)").strip("\n")
        return int(float(n))
    
    
    def collect_buffer(instr, buffer_nr=1, verbose=False):
        """
        Get the buffer as 2D - np.array
        @param instr : pyvisa instrument
        @param buffer_nr : 1 or 2, for smua.nvbuffer1 or 2
        @returns 2D numpy array:
            i - ith reading:
                0: timestamps
                1: readings
        """
        buffername = get_buffer_name(buffer_nr)
        # instr.write("format.data = format.DREAL\nformat.byteorder = format.LITTLEENDIAN")
        # buffer = instr.query_binary_values(f"printbuffer(1, {buffername}.n, {buffername})", datatype='d', container=np.array)
        instr.write("format.data = format.ASCII\nformat.asciiprecision = 7")
        timestamps = instr.query_ascii_values(f"printbuffer(1, {buffername}.n, {buffername}.timestamps)", container=np.array)
        readings = instr.query_ascii_values(f"printbuffer(1, {buffername}.n, {buffername}.readings)", container=np.array)
        if verbose:
            print(f"readings from {buffername}: {readings}, \ntimestamps: {timestamps}")
        buffer = np.vstack((timestamps, readings)).T
        return buffer
    
    
    
    def collect_buffer_range(instr, range_=(1, -1), buffer_nr=1, verbose=False):
        """
        Get the buffer as 2D - np.array
        @param instr : pyvisa instrument
        @param buffer_nr : 1 or 2, for smua.nvbuffer1 or 2
        @returns 2D numpy array:
            i - ith reading:
                0: timestamps
                1: readings
        """
        buffername = get_buffer_name(buffer_nr)
        # instr.write("format.data = format.DREAL\nformat.byteorder = format.LITTLEENDIAN")
        # buffer = instr.query_binary_values(f"printbuffer(1, {buffername}.n, {buffername})", datatype='d', container=np.array)
        if range_[1] == -1:
            range_ = (range_[0], f"{buffername}.n")
        instr.write("format.data = format.ASCII\nformat.asciiprecision = 7")
        timestamps = instr.query_ascii_values(f"printbuffer({range_[0]}, {range_[1]}, {buffername}.timestamps)", container=np.array)
        readings = instr.query_ascii_values(f"printbuffer({range_[0]}, {range_[1]}, {buffername}.readings)", container=np.array)
        if verbose:
            print(f"readings from {buffername}: {readings}, \ntimestamps: {timestamps}")
        buffer = np.vstack((timestamps, readings)).T
        return buffer
    
    
    
    
    
